/* Copyright (C) 2008 Vin Helminen
 *$Id: cache.c,v 1.1.1.1 2008-07-23 23:30:02 vah Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "myerror.h"
#include "cache.h"

char* dupstr(const char *src)
{
  size_t len = strlen(src)+1;
  char *dst = malloc(len);
  if(dst) memcpy(dst, src, len);
  return dst;
}


/* Use fast XOR based hash function */
#define FAST_HASH 1

/* How many bits of the hash will be used. FAST_HASH only has maximum of 8 bits */
#define HASH_BITS 4
#define HASH_MASK ((1<<HASH_BITS)-1)
#define ARRAY_SIZE (1 << HASH_BITS)

static unsigned getHash(const char *str)
{
  unsigned hash = 0, c;
  while((c = *str++)) {
#if FAST_HASH
    hash  = c + (hash << 6) + (hash << 16) - hash;
#else
    hash ^= c;
#endif
  }
  return hash & HASH_MASK;
}

struct CacheItem {
  char *key;
  void *value;
  struct CacheItem *next;
};

Cache* initCache(void)
{
  Cache *cache;
  cache = malloc(ARRAY_SIZE * sizeof *cache);

  CHECK_MALLOC(cache);

  for(int i = 0; i < ARRAY_SIZE; ++i)
    cache[i] = NULL;

  return cache;
}

void freeCache(Cache *cache, FreeFunc freefunc)
{
  for(int i = 0; i < ARRAY_SIZE; ++i) {
    struct CacheItem *item = cache[i];
    while(item) {
      struct CacheItem *next = item->next;
      if(freefunc) freefunc(item->key, item->value);
      free(item->key);
      free(item);
      item = next;
    }
  }
  free(cache);
}

void addToCache(Cache *cache, const char *key, void *value)
{
  struct CacheItem *item;
  item = malloc(sizeof *item);
  CHECK_MALLOC(item);

  item->key = dupstr(key);
  CHECK_MALLOC(item->key);

  item->value = value;

  unsigned hash = getHash(key);
  if(cache[hash]) item->next = cache[hash];
  else item->next = NULL;
  cache[hash] = item;
}

void* getFromCache(const Cache *cache, const char *key)
{
  for(struct CacheItem *item = cache[getHash(key)]; item; item = item->next) {
    if(0 == strcmp(key, item->key))
      return item->value;
  }
  return NULL;
}
