#ifndef CACHE_H
#define CACHE_H
/* Copyright (C) 2008 Vin Helminen
 *$Id: cache.h,v 1.1.1.1 2008-07-23 23:30:02 vah Exp $ */
#include <string.h>

struct CacheItem;

typedef struct CacheItem* Cache;

typedef void (FreeFunc)(const char*, void*);

/* duplicate a string (see man strdup) */
char* dupstr(const char *src);

/* allocate and initalize new cache */
Cache* initCache(void);

/* free the cache structure
 * calls freefunc (if not NULL) for each item to free the stored values */
void freeCache(Cache *cache, FreeFunc freefunc); 

/* add a new item to the cache
 * this will not check for duplicate keys and getFromCache() returns the first one it finds (=last one added)
 * the key is copied and freed when freeCache() is called */
void addToCache(Cache *cache, const char *key, void *value);

/* find the value for a key
 * returns NULL if key was not found */
void* getFromCache(const Cache *cache, const char *key);

#endif
