/*
 * Copyright (C) 2006-2008 Vin Helminen 
 * $Id: data.c,v 1.1.1.1 2008-07-23 23:30:02 vah Exp $
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <libgen.h>

#include "data.h"

#ifndef PATH_MAX
#define PATH_MAX 4096
#endif

#define SSTR(x) #x
#define STR(x) SSTR(x)

static char* getDataDir(void)
{
  char path[PATH_MAX + 1];
#ifdef DATA_PATH
  /* use predefined data path */
  strncpy(path, STR(DATA_PATH), PATH_MAX);

#else
  /* use relative path to the executable */
  ssize_t ret = readlink("/proc/self/exe", path, PATH_MAX);
  if(ret > 0) {
    path[ret] = '\0';
    /* get just the directory */
    char *dir = dirname(path);
    if(dir != path) strcpy(path, dir);
  } else {
    /* readlink() failed. hope that current directory is fine... */
    path[0] = '.';
    path[1] = '\0';
  }
#endif

  char *resolved = realpath(path, NULL);
  if(!resolved) return NULL;

  size_t len = strlen(resolved);
  resolved[len] = '/';
  resolved[len+1] = '\0';
  resolved = realloc(resolved, len+2);

  return resolved;
}

const char* getFilePath(const char *filename)
{
  static char *dir = NULL;
  static char *path = NULL;
  if(filename[0] == '/')
    return filename;
  if(!dir) dir = getDataDir();
  if(!path) path = malloc(PATH_MAX+1);
  if(!dir || !path) return filename;
  strcpy(path, dir);
  return strcat(path, filename);  
}
