/*
 * Copyright (C) 2006-2008 Vin Helminen 
 * $Id: time.c,v 1.1.1.1 2008-07-23 23:30:02 vah Exp $
 */

#include "myerror.h"
#include "time.h"

/* frame rate control */

static Uint32 frametime, start, ticks, frames;

void initTime(int framespersecond)
{
  if(framespersecond < 1)
    framespersecond = 1;
  else if(framespersecond > MAX_FRAME_RATE)
    framespersecond = MAX_FRAME_RATE;
  frametime = 1000/framespersecond;
  start = ticks = SDL_GetTicks();
  frames = 1;
  SDL_Delay(frametime);
}

unsigned tick(void)
{
  Uint32 renderTime = ticks;
  ticks = SDL_GetTicks();
  renderTime = ticks - renderTime;

  // the difference between virtual time and real time
  int diff = (frametime * frames) - (ticks - start);
  unsigned step = 1;

  if(diff > 0) {
    // sleep if there is plenty of time to spend
    //if(diff >= 10)
    SDL_Delay(diff);
  } else {
    // we're too slow, have to skip few frames
    for(; diff < 0; diff += frametime)
      ++step;
  }

  frames += step;

  return step;
}

/* stop clock */

#define CURRENT_TIME (frames * frametime)

static Uint32 clockStart = 0, clockStop = 0;

void startClock(void)
{
  clockStart = CURRENT_TIME;
  clockStop = 0;
}

void stopClock(void)
{
  if(!clockStop)
    clockStop = CURRENT_TIME;
}

void resetClock(void)
{
  clockStart = clockStop = 0;
}

Uint32 getClock(void)
{
  if(clockStop)
    return clockStop - clockStart;
  if(clockStart)
    return CURRENT_TIME - clockStart;
  return 0;
}
