#ifndef VIDEO_H
#define VIDEO_H
/*
 * Copyright (C) 2006-2008 Vin Helminen 
 * $Id: video.h,v 1.1.1.1 2008-07-23 23:30:02 vah Exp $
 */

#include <stdbool.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "sysconf.h"

#define TEXTURE_SIZE_EXP 8
#define TEXTURE_SIZE (1<<TEXTURE_SIZE_EXP)
#define TEXTURE_SIZE_MASK (TEXTURE_SIZE-1)

#if SCREEN_BPP == 32
typedef Uint32 Pixel;

static inline void getRGB(Pixel pixel, unsigned *r, unsigned *g, unsigned *b)
{
  *r = (pixel >> 16) & 0xFF;
  *g = (pixel >> 8) & 0xFF;
  *b = (pixel) & 0xFF;
}
static inline Pixel setRGB(unsigned r, unsigned g, unsigned b)
{
  return ((r << 16) & 0xFF0000) | ((g << 8) & 0xFF00) | (b &0xFF);
}
#endif

#if SCREEN_BPP == 16
typedef Uint16 Pixel;

static inline void getRGB(Pixel pixel, unsigned *r, unsigned *g, unsigned *b)
{
  *r = (pixel >> 8) & 0xF8;
  *g = (pixel >> 3) & 0xFC;
  *b = (pixel << 3) & 0xF8;
}
static inline Pixel setRGB(unsigned r, unsigned g, unsigned b)
{
  return ((r << 8) & 0xF800) | ((g << 3) & 0x7E0) | ((b >> 3) &0x1F);
}
#endif

#if SCREEN_BPP == 8
typedef Uint8 Pixel;

static inline void getRGB(Pixel pixel, unsigned *r, unsigned *g, unsigned *b)
{
  *r = pixel & 0xE0;
  *g = (pixel << 3) & 0xE0;
  *b = ((pixel << 6) & 0xC0) | ((pixel << 4) & 0x20);
}
static inline Pixel setRGB(unsigned r, unsigned g, unsigned b)
{
  return (r & 0xE0) | ((g >> 3) & 0x1C) | ((b >> 6) & 0x3);
}
#endif

SDL_Surface* initVideo(bool fullscreen);

void flip(void);

/* load an image converted to display format with optional scaling */
SDL_Surface* loadImage(const char *filename, int width, int height);

/* Textures */

const Pixel* loadTexture(const char *filename);

/* Sprites and text */

typedef struct Sprite {
  SDL_Surface *surface;
  int width, height; // dimensions of a single sprite not the whole surface's which can contain many
  int offset; // index offset. used with fonts
} Sprite;

const Sprite* loadSprites(const char *filename, int width, int height, int offset);

void drawSprite(const Sprite *sprite, int index, int x, int y, SDL_Surface *surface);

/* x >= 0 is aligned left,  x < 0 is aligned right */
void drawString(const Sprite *font, const char *text, int x, int y, SDL_Surface *surface);


/* SLOW but extremely compatible graphic functions for preprocessing */

/* parameters are: surface, x, y, color */
typedef void (*SetPixelFunc)(SDL_Surface*, int, int, SDL_Color);

/* parameters are: surface, x, y */
typedef SDL_Color (*GetPixelFunc)(const SDL_Surface*, int, int);

/* Scale src surface to fill dst surface. (Does not keep aspect ratio.)
 * Anti-aliases when scaling down in either or both dimensions. */
void scaleSurface(const SDL_Surface *src, SDL_Surface *dst);

/* get setPixel function suitable for the surface */
SetPixelFunc getSetPixel(SDL_Surface *surface);

/* get getPixel function suitable for the surface */
GetPixelFunc getGetPixel(const SDL_Surface *surface);



#endif
